@@ -149,8 +149,13 @@ llqr <- function(x, y, tau=0.5, h = NULL, method="rule", x0 = NULL) {
       h <- llqrcv(x, y, tau)
     }
     if (method == "rule") {
-      h <- KernSmooth::dpill(x, y)
-      h <- h * (tau * (1 - tau) / (dnorm(qnorm(tau)))^2)^.2
+      red_dim <- floor(0.2 * n) # find how many observations correspond to a 20%
+      index_y <- order(y)[red_dim:(n - red_dim)] # subtract the smallest 20% and the largest 20% of the observations
+      h <- KernSmooth::dpill(x[index_y, ], y[index_y])
+      h <- 1.25 * h * (tau * (1 - tau) / (dnorm(qnorm(tau)))^2)^.2
+      if (h == 'NaN') {
+        h <- 1.25 * max(n^(-1 / (p + 4)), min(2, sd(y)) * n^(- 1 / (p + 4)))
+      }
     }
   } else {
     h <- h
